from textual.theme import Theme

retro_theme = Theme(
    name="retro-green",
    primary="#00ff00",
    secondary="#00cc00",
    accent="#00dd00",
    foreground="#00ff00",
    background="#000000",
    surface="#002200",
    panel="#003300",
    success="#55ff55",
    warning="#ffff00",
    error="#ff5555",
    dark=True,
)
